@@ -163,13 +163,40 @@ kSyncFrame
 
 // TYPES
 
+typedef UInt16 USBDeviceAddress;
+
+
+/*!
+    @typedef IOUSBIsocFrame
+    Structure used to encode information about each isoc frame
+    @param frStatus Returns status associated with the frame
+    @param frReqCount Input specifiying how many bytes to read or write
+    @param frActCount Actual # of bytes transferred
+*/
 typedef struct IOUSBIsocFrame {
     IOReturn                        frStatus;
     UInt16                          frReqCount;
     UInt16                          frActCount;
 } IOUSBIsocFrame;
 
-typedef UInt16 USBDeviceAddress;
+
+/*!
+    @typedef                         
+    Structure used to encode information about each isoc frame that is processed
+    at hardware interrupt time
+    @param frStatus Returns status associated with the frame
+    @param frReqCount Input specifiying how many bytes to read or write
+    @param frActCount Actual # of bytes transferred
+    @param frTimeStamp Time stamp that indicates time when frame was procesed
+*/
+struct IOUSBLowLatencyIsocFrame {
+    IOReturn                        frStatus;
+    UInt16                          frReqCount;
+    UInt16                          frActCount;
+    AbsoluteTime		    frTimeStamp;
+};
+
+typedef struct IOUSBLowLatencyIsocFrame IOUSBLowLatencyIsocFrame;
 
 /*!
     @typedef IOUSBCompletionAction
@@ -191,6 +218,12 @@ typedef void (*IOUSBIsocCompletionAction)(
                 IOReturn		status,
                 IOUSBIsocFrame		*pFrames);
 
+typedef void (*IOUSBLowLatencyIsocCompletionAction)(
+                void *				target,
+                void *				parameter,
+                IOReturn			status,
+                IOUSBLowLatencyIsocFrame	*pFrames);
+
 /*!
     @typedef IOUSBCompletion
     Struct spefifying action to perform when a USB I/O completes
@@ -211,6 +244,14 @@ typedef struct IOUSBIsocCompletion {
     void *			parameter;
 } IOUSBIsocCompletion;
 
+typedef struct IOUSBLowLatencyIsocCompletion {
+    void * 				target;
+    IOUSBLowLatencyIsocCompletionAction	action;
+    void *				parameter;
+};
+
+typedef struct IOUSBLowLatencyIsocCompletion IOUSBLowLatencyIsocCompletion;
+
 /* ************* USB Family error codes ************* */
 
 #define	iokit_usb_err(return) (sys_iokit|sub_iokit_usb|return)
@@ -380,6 +421,35 @@ struct IOUSBHIDReportDesc {
 typedef struct IOUSBHIDReportDesc	IOUSBHIDReportDesc;
 typedef IOUSBHIDReportDesc *		IOUSBHIDReportDescPtr;
 
+#pragma pack(1)
+struct IOUSBDeviceQualifierDescriptor
+{
+	UInt8 			bLength;
+	UInt8 			bDescriptorType;
+	UInt16 			bcdUSB;
+	UInt8 			bDeviceClass;
+	UInt8 			bDeviceSubClass;
+	UInt8 			bDeviceProtocol;
+	UInt8 			bMaxPacketSize0;
+	UInt8 			bNumConfigurations;
+	UInt8 			bReserved;
+};
+typedef struct IOUSBDeviceQualifierDescriptor	IOUSBDeviceQualifierDescriptor;
+typedef IOUSBDeviceQualifierDescriptor *	IOUSBDeviceQualifierDescriptorPtr;
+
+struct IOUSBDFUDescriptor 
+{
+        UInt8 			bLength;
+        UInt8 			bDescriptorType;
+        UInt8 			bmAttributes;
+        UInt16 			wDetachTimeout;
+        UInt16 			wTransferSize;
+};
+typedef struct 	IOUSBDFUDescriptor 		IOUSBDFUDescriptor;
+typedef 		IOUSBDFUDescriptor *	IOUSBDFUDescriptorPtr;
+
+#pragma options align=reset
+
 typedef UInt16				USBStatus;
 typedef USBStatus *			USBStatusPtr;
 
@@ -548,6 +618,20 @@ typedef struct {
     IOUSBIsocFrame 	*fFrameCounts;
 } IOUSBIsocStruct;
 
+// Structure to request low latency isochronous transfer
+//
+struct IOUSBLowLatencyIsocStruct {
+    UInt32 			fPipe;
+    void *			fBuffer;
+    UInt32 			fBufSize;
+    UInt64 			fStartFrame;
+    UInt32 			fNumFrames;
+    UInt32			fUpdateFrequency;
+    IOUSBLowLatencyIsocFrame *	fFrameCounts;
+};
+
+typedef struct IOUSBLowLatencyIsocStruct IOUSBLowLatencyIsocStruct;
+
 // Structure to return frame number, and time frame register was read.
 //
 typedef struct {
@@ -577,6 +661,10 @@ enum {
         kUSBDeviceSpeedHigh		= 2			// high speed device
         };
 
+enum {
+        kUSBLowLatencyIsochTransferKey	= 'llit'	// Set frStatus field of first frame in isoch transfer to designate as low latency
+    };
+    
 // misc. properties which are useful
 #define	kUSBDevicePropertySpeed			"Device Speed"
 #define kUSBDevicePropertyBusPowerAvailable 	"Bus Power Available"
