@@ -88,6 +88,9 @@ enum {
     kUSBInterfaceUserClientAsyncWritePipe,
     kUSBInterfaceUserClientReadIsochPipe,
     kUSBInterfaceUserClientWriteIsochPipe,
+    // new with 1.9.2
+    kUSBInterfaceUserClientLowLatencyReadIsochPipe,
+    kUSBInterfaceUserClientLowLatencyWriteIsochPipe,
     kNumUSBInterfaceAsyncMethods
     };
 
